/* Ripped from another code of mine :) */
#include <stdio.h>
#include <stdlib.h>
#include <ncurses.h>
#include <sys/time.h>
#include <time.h>
#include <stdarg.h>
#include "gui.h"

int UI_InitBrowser(Browser *br, int X, int Y, int Width, int Height, int Color)
{
   int x,y,i;
   
   br->win = newwin(Height,Width,Y,X);
   if (br->win == NULL)
     return -1;
   
   br->x = Width;
   br->y = Height;
   br->color = Color;
   x = (br->x - 2) * (br->y - 2); // window size in bytes
   br->content = malloc(x);
   memset(br->content,0,x);
   
   wattrset(br->win, COLOR_PAIR(br->color));
   for (x=0;x<Width;x++)
     for (y=0;y<Height;y++) mvwaddch(br->win, y, x, ' ');
   
   box(br->win,ACS_VLINE,ACS_HLINE);
}

void UI_AddBrowser(Browser *br, const char *str, ...)
{
   const int line = br->x - 2;
   char buf[line], *ptr;
   va_list args;
   int i;
   
   va_start(args,str);
   vsnprintf(buf,line,str,args);
   
   // try to place buf in an empty line, if available
   for (i=0;i<(br->y - 2);i++) {
      ptr = br->content + i*line;
      if (ptr[0] == 0x0) {
	 memset(ptr,0,line);
	 memcpy(ptr,buf,strlen(buf));
	 return;
      }
   }
   
   // no empty lines, clean up some space and place it on the last line
   for (i=1;i<(br->y - 2);i++)
     memcpy(br->content + (i-1)*line,br->content + i*line, line);
   
   ptr = br->content + (br->y - 3)*line;
   memset(ptr,0,line);
   memcpy(ptr, buf, strlen(buf));
}

// put content in browser window and render
void UI_RenderBrowser(Browser *br)
{
   const int line = br->x - 2;
   int x, y;
   
   wattrset(br->win,COLOR_PAIR(br->color));
   for (y=0;y<(br->y - 2);y++)
     for (x=0;x<line;x++) {
	char *ptr = br->content + y*line + x;
	mvwaddch(br->win, y+1, x+1, (ptr[0] == 0x0 || ptr[0] == 0xa)?' ':ptr[0]);
     }
   wrefresh(br->win);
}

void UI_KillBrowser(Browser *br)
{
   free(br->content);
   delwin(br->win);
}

char* UI_Input(int x, int y, int n, int color)
{
   WINDOW *win;
   char *input;
   int i;
   
   if ((win = newwin(1,n,y,x)) == NULL || (input = malloc(n)) == NULL)
     return NULL;
   
   wattrset(win, COLOR_PAIR(color));
   for (i=0;i<n;i++) mvwaddch(win, 0, i, ' ');
   memset(input, 0, n);
   mvwaddch(win, 0, 0, '>');
   wrefresh(win);
   mvwgetnstr(win, 0, 1, input, n);
   delwin(win);   
   return input;
}

void UI_Init(void)
{
   initscr();
   start_color();
   cbreak();
   
   init_pair(1, COLOR_WHITE, COLOR_BLUE);
   init_pair(2, COLOR_BLACK, COLOR_WHITE);
}

void UI_End(void)
{
   endwin();
}
