#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <elf.h>
#include <fcntl.h>
#include "segment.h"

/* see elf.h */
struct Description types[SEGMENT_TYPES] =
{
   {"Program header table entry",    PT_NULL},
   {"Loadable program segment",      PT_LOAD},
   {"Dynamic linking information",PT_DYNAMIC},
   {"Program interpreter",         PT_INTERP},
   {"Auxiliary information",         PT_NOTE},
   {"Reserved",                     PT_SHLIB},
   {"Entry for header table itself", PT_PHDR},
   {"Number of defined types",        PT_NUM},
   {"Start of OS-specific",          PT_LOOS},
   {"End of OS-specific",            PT_HIOS},
   {"Start of processor-specific", PT_LOPROC},
   {"End of processor-specific",   PT_HIPROC}
};

char *Segment_type(Elf32_Word p_type)
{
   int i;
   char *ptr = NULL;
   
   for (i=0;i<SEGMENT_TYPES;i++)
     if (types[i].num == p_type) {
	ptr = types[i].name;
	break;
     }
   return ptr;
}

int Segment_valid(Elf32_Word p_type)
{
   int i;
   for (i=0;i<SEGMENT_TYPES;i++)
     if (types[i].num == p_type) return 0;
   return -1;
}

int Segment_init(int fd)
{
   Elf32_Ehdr hdr;
   
   lseek(fd, 0, SEEK_SET);
   read(fd, &hdr, sizeof(Elf32_Ehdr));
   if (hdr.e_type == 4) return 0;
   return -1;
}
